/*

    Copyright (c) 2018 - 2020 Analog Flavor EURL

    Contact us for support:
        http://www.analogflavor.com

-----------------------------------------------------------------------------
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

    * Neither the name of Analog Flavor EURL nor 
      the names of its contributors may be used to endorse or promote products derived from
      this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
-----------------------------------------------------------------------------

*/

#include "wiener_mpod_lv_hv_snmp.h"
#include "WienerMpodLvHvSnmp.h"

#ifdef WIN32
#include <WinSock2.h>
#endif

void *mpod_snmp_create() {
    WienerMpodLvHvSnmp *obj = new WienerMpodLvHvSnmp();
    return obj;
}

void mpod_snmp_delete(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        delete mpod_snmp_obj;
    }
}

int mpod_snmp_crateConnect(void *obj_ptr, const char *ip_address, int port_index, const char *community_name, int thread_polling_interval_ms_fast, int thread_polling_interval_ms_intermediate, int thread_polling_interval_ms_slow) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->crateConnect(ip_address, port_index, community_name, thread_polling_interval_ms_fast, thread_polling_interval_ms_intermediate, thread_polling_interval_ms_slow);
    }
    return 0;
}

void mpod_snmp_crateDisconnect(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        mpod_snmp_obj->crateDisconnect();
    }
}

mpod_snmp_connection_status_type mpod_snmp_crateGetConnectionStatus(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->lockAndGetCrateConnectionStatus();
    }
    return mpod_snmp_connection_status_undefined;
}

const char *mpod_snmp_crateGetErrorString(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->lockAndGetErrorString());
    }
    return 0;
}

const char *mpod_snmp_crateGetConnectionStatusAsString(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->lockAndGetCrateConnectionStatusAsString());
    }
    return 0;
}

int mpod_snmp_crateCheckConnectionIsOk(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->lockAndCheckCrateConnectionIsOk();
    }
    return 0;
}

int mpod_snmp_checkIsMibFileLoaded(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->checkIsMibFileLoaded();
    }
    return 0;
}

void mpod_snmp_snmpSetTimeoutInterval(void *obj_ptr, int interval_in_ms) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        mpod_snmp_obj->snmpSetTimeoutInterval(interval_in_ms);
    }
}

double mpod_snmp_channelGetConfigMaxCurrent(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetConfigMaxCurrent(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetConfigMaxSenseVoltage(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetConfigMaxSenseVoltage(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetConfigMaxTerminalVoltage(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetConfigMaxTerminalVoltage(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetCurrent(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetCurrent(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetCurrentFallRate(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetCurrentFallRate(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetCurrentRiseRate(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetCurrentRiseRate(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

int mpod_snmp_channelGetIndex(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetIndex(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

double mpod_snmp_channelGetMeasurementCurrent(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetMeasurementCurrent(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetMeasurementSenseVoltage(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetMeasurementSenseVoltage(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetMeasurementTemperature(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetMeasurementTemperature(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetMeasurementTerminalVoltage(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetMeasurementTerminalVoltage(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

int mpod_snmp_channelGetStatus(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetStatus(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

const char *mpod_snmp_channelGetStatusAsString(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->channelGetStatusAsString(slot_index, channel_index));
    }
    return 0;
}

int mpod_snmp_channelGetSupervisionBehavior(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetSupervisionBehavior(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

double mpod_snmp_channelGetSupervisionMaxCurrent(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetSupervisionMaxCurrent(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetSupervisionMaxPower(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetSupervisionMaxPower(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetSupervisionMaxTerminalVoltage(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetSupervisionMaxTerminalVoltage(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetSupervisionMinSenseVoltage(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetSupervisionMinSenseVoltage(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

int mpod_snmp_channelGetSwitch(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetSwitch(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelGetTripTime(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetTripTime(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelGetUserConfig(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetUserConfig(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

double mpod_snmp_channelGetVoltage(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetVoltage(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetVoltageFallRate(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetVoltageFallRate(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_channelGetVoltageRiseRate(void *obj_ptr, unsigned slot_index, unsigned channel_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelGetVoltageRiseRate(slot_index, channel_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

int mpod_snmp_channelSetCurrent(void *obj_ptr, unsigned slot_index, unsigned channel_index, double value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetCurrent(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelSetCurrentFallRate(void *obj_ptr, unsigned slot_index, unsigned channel_index, double value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetCurrentFallRate(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelSetCurrentRiseRate(void *obj_ptr, unsigned slot_index, unsigned channel_index, double value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetCurrentRiseRate(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelSetSupervisionBehavior(void *obj_ptr, unsigned slot_index, unsigned channel_index, int value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetSupervisionBehavior(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelSetSupervisionMaxCurrent(void *obj_ptr, unsigned slot_index, unsigned channel_index, double value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetSupervisionMaxCurrent(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelSetSupervisionMaxTerminalVoltage(void *obj_ptr, unsigned slot_index, unsigned channel_index, double value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetSupervisionMaxTerminalVoltage(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelSetSupervisionMinSenseVoltage(void *obj_ptr, unsigned slot_index, unsigned channel_index, double value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetSupervisionMinSenseVoltage(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelSetSwitch(void *obj_ptr, unsigned slot_index, unsigned channel_index, int value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetSwitch(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelSetTripTime(void *obj_ptr, unsigned slot_index, unsigned channel_index, int value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetTripTime(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelSetUserConfig(void *obj_ptr, unsigned slot_index, unsigned channel_index, int value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetUserConfig(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelSetVoltage(void *obj_ptr, unsigned slot_index, unsigned channel_index, double value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetVoltage(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelSetVoltageFallRate(void *obj_ptr, unsigned slot_index, unsigned channel_index, double value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetVoltageFallRate(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_channelSetVoltageRiseRate(void *obj_ptr, unsigned slot_index, unsigned channel_index, double value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->channelSetVoltageRiseRate(slot_index, channel_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_crateGetFanSpeed(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->crateGetFanSpeed();
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_crateGetNumberOfChannels(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->crateGetNumberOfChannels();
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_crateGetNumberOfGroups(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->crateGetNumberOfGroups();
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_crateGetNumberOfModules(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->crateGetNumberOfModules();
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_crateGetNumberOfOccupiedSlots(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->crateGetOccupiedSlots().size();
    }
    return 0;
}

const int *mpod_snmp_crateGetOccupiedSlots(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCArray(mpod_snmp_obj->crateGetOccupiedSlots());
    }
    return 0;
}
const char *mpod_snmp_crateGetSerialNumber(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->crateGetSerialNumber());
    }
    return 0;
}

const char *mpod_snmp_crateGetSysDescr(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->crateGetSysDescr());
    }
    return 0;
}

int mpod_snmp_crateGetSysMainSwitch(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->crateGetSysMainSwitch();
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_crateGetSysStatus(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->crateGetSysStatus();
    }
    return WienerMpodLvHvSnmp::getIntError();
}

const char *mpod_snmp_crateGetSysStatusAsString(void *obj_ptr) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->crateGetSysStatusAsString());
    }
    return 0;
}

int mpod_snmp_crateSetFanSpeed(void *obj_ptr, int value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->crateSetFanSpeed(value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_crateSetSysHardwareReset(void *obj_ptr, int value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->crateSetSysHardwareReset(value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_crateSetSysMainSwitch(void *obj_ptr, int value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->crateSetSysMainSwitch(value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_groupGetIndex(void *obj_ptr, unsigned index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->groupGetIndex(index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_groupSetSwitch(void *obj_ptr, unsigned group_index, int value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->groupSetSwitch(group_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

double mpod_snmp_moduleGetAuxiliaryMeasurementVoltage0(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetAuxiliaryMeasurementVoltage0(slot_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_moduleGetAuxiliaryMeasurementVoltage1(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetAuxiliaryMeasurementVoltage1(slot_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

const char *mpod_snmp_moduleGetDescription(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->moduleGetDescription(slot_index));
    }
    return 0;
}

int mpod_snmp_moduleGetEventChannelStatus(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetEventChannelStatus(slot_index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_moduleGetEventStatus(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetEventStatus(slot_index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

const char *mpod_snmp_moduleGetEventStatusAsString(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->moduleGetEventStatusAsString(slot_index));
    }
    return 0;
}

double mpod_snmp_moduleGetHardwareLimitCurrent(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetHardwareLimitCurrent(slot_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_moduleGetHardwareLimitVoltage(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetHardwareLimitVoltage(slot_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

int mpod_snmp_moduleGetIndex(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetIndex(slot_index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_moduleGetIsHvModule(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetIsHvModule(slot_index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

double mpod_snmp_moduleGetMeasurementTemperature0(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetMeasurementTemperature0(slot_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_moduleGetMeasurementTemperature1(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetMeasurementTemperature1(slot_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_moduleGetMeasurementTemperature2(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetMeasurementTemperature2(slot_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_moduleGetMeasurementTemperature3(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetMeasurementTemperature3(slot_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

int mpod_snmp_moduleGetNumberOfChannels(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetNumberOfChannels(slot_index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

double mpod_snmp_moduleGetRampSpeedCurrent(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetRampSpeedCurrent(slot_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

double mpod_snmp_moduleGetRampSpeedVoltage(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetRampSpeedVoltage(slot_index);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

int mpod_snmp_moduleGetStatus(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleGetStatus(slot_index);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

const char *mpod_snmp_moduleGetStatusAsString(void *obj_ptr, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->moduleGetStatusAsString(slot_index));
    }
    return 0;
}

int mpod_snmp_moduleSetDoClear(void *obj_ptr, unsigned slot_index, int value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleSetDoClear(slot_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_moduleSetRampSpeedCurrent(void *obj_ptr, unsigned slot_index, double value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleSetRampSpeedCurrent(slot_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

int mpod_snmp_moduleSetRampSpeedVoltage(void *obj_ptr, unsigned slot_index, double value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->moduleSetRampSpeedVoltage(slot_index, value);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

const char *mpod_snmp_snmpGetDescription(void *obj_ptr, const char *string_oid) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->snmpGetDescription(string_oid));
    }
    return 0;
}

const char *mpod_snmp_snmpGetNextOid(void *obj_ptr, const char *string_oid) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->snmpGetNextOid(string_oid));
    }
    return 0;
}

double mpod_snmp_snmpGetValueDouble(void *obj_ptr, const char *string_oid) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->snmpGetValueDouble(string_oid);
    }
    return WienerMpodLvHvSnmp::getDoubleError();
}

int mpod_snmp_snmpGetValueInt(void *obj_ptr, const char *string_oid) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->snmpGetValueInt(string_oid);
    }
    return WienerMpodLvHvSnmp::getIntError();
}

const char *mpod_snmp_snmpGetValueString(void *obj_ptr, const char *string_oid) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->snmpGetValueString(string_oid));
    }
    return 0;
}

int mpod_snmp_snmpSetValueDouble(void *obj_ptr, const char *string_oid, double value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->snmpSetValueDouble(string_oid, value);
    }
    return 0;
}

int mpod_snmp_snmpSetValueInt(void *obj_ptr, const char *string_oid, int value) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->snmpSetValueInt(string_oid, value);
    }
    return 0;
}

const char *mpod_snmp_snmpTranslateFromOid(void *obj_ptr, const char *string_oid) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->snmpTranslateFromOid(string_oid));
    }
    return 0;
}

const char *mpod_snmp_snmpTranslateToOid(void *obj_ptr, const char *string_oid) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->snmpTranslateToOid(string_oid));
    }
    return 0;
}

const char *mpod_snmp_convertCrateStatusToString(void *obj_ptr, int integer_status) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->convertCrateStatusToString(integer_status));
    }
    return 0;
}

const char *mpod_snmp_convertModuleEventStatusToString(void *obj_ptr, int integer_status) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->convertModuleEventStatusToString(integer_status));
    }
    return 0;
}

const char *mpod_snmp_convertModuleStatusToString(void *obj_ptr, int integer_status) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->convertModuleStatusToString(integer_status));
    }
    return 0;
}

const char *mpod_snmp_convertChannelStatusToString(void *obj_ptr, int integer_status, unsigned slot_index) {
    if (obj_ptr) {
        WienerMpodLvHvSnmp *mpod_snmp_obj = (WienerMpodLvHvSnmp*)(obj_ptr);
        return mpod_snmp_obj->toCString(mpod_snmp_obj->convertChannelStatusToString(integer_status, slot_index));
    }
    return 0;
}

#ifdef WIN32
static bool flag_winsock_startup_was_called = false;
int mpod_snmp_winsock_startup() {
    if (!flag_winsock_startup_was_called) {
        flag_winsock_startup_was_called = true;
        WSADATA WinsockData;
        int res = WSAStartup(MAKEWORD(2, 0), &WinsockData);
        if (res != 0) {
            return 0;
        }
    }
    return 1;
}
void mpod_snmp_winsock_cleanup() {
    if (flag_winsock_startup_was_called) {
        WSACleanup();
    }
}
#else
int mpod_snmp_winsock_startup() {
    return 1;
}
void mpod_snmp_winsock_cleanup() {
}
#endif
