/*

    Copyright (c) 2018 - 2020 Analog Flavor EURL

    Contact us for support:
        http://www.analogflavor.com

-----------------------------------------------------------------------------
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

    * Neither the name of Analog Flavor EURL nor 
      the names of its contributors may be used to endorse or promote products derived from
      this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
-----------------------------------------------------------------------------

*/



#include <wiener_mpod_lv_hv_snmp.h>

#include <string.h>
#include <math.h>
#include <stdio.h>
#ifdef WIN32
#include <Windows.h>
void sleep_ms(int ms) {
    Sleep(ms);
}
#else
#include <unistd.h>
void sleep_ms(int ms) {
    usleep(ms * 1000);
}
#endif

int main(int argc, char **argv) {

    void *obj_ptr;
    const char *string_value;
    double double_value;
    int int_value;
    unsigned module_slot_index, channel_index;
    int thread_polling_interval_ms = 1000; /* set to 0 for debug */

    if (!mpod_snmp_winsock_startup()) {
        printf("Winsock startup failed.\n");
        getchar();
        return 1;
    }

    /* create object */
    obj_ptr = mpod_snmp_create();

    /*  establish connection */
    if (argc >= 2) {
        /* the port ans community can also specified as  "192.168.0.101:161:guru" only in the first argument. */
        mpod_snmp_crateConnect(obj_ptr, argv[1], 0, "", 0.5 * thread_polling_interval_ms, thread_polling_interval_ms, 2.0 * thread_polling_interval_ms);
    }
    else {
        mpod_snmp_crateConnect(obj_ptr, "192.168.0.101", 161, "guru", 0.5 * thread_polling_interval_ms, thread_polling_interval_ms, 2.0 * thread_polling_interval_ms);
    }

    /* check that the connection has been established */
    if (mpod_snmp_crateCheckConnectionIsOk(obj_ptr)) {

        /* make sure all data has been read to the buffer */
        sleep_ms(3 * thread_polling_interval_ms);

        /* call functions to get and set SNMP variables */

        module_slot_index = 3;
        channel_index = 0;

        double_value = mpod_snmp_channelGetConfigMaxCurrent(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetConfigMaxCurrent() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetConfigMaxSenseVoltage(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetConfigMaxSenseVoltage() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetConfigMaxTerminalVoltage(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetConfigMaxTerminalVoltage() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetCurrent(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetCurrent() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetCurrentFallRate(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetCurrentFallRate() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetCurrentRiseRate(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetCurrentRiseRate() returns %f.\n", double_value);

        int_value = mpod_snmp_channelGetIndex(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetIndex() returns %d.\n", int_value);

        double_value = mpod_snmp_channelGetMeasurementCurrent(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetMeasurementCurrent() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetMeasurementSenseVoltage(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetMeasurementSenseVoltage() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetMeasurementTemperature(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetMeasurementTemperature() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetMeasurementTerminalVoltage(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetMeasurementTerminalVoltage() returns %f.\n", double_value);

        int_value = mpod_snmp_channelGetStatus(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetStatus() returns 0x%x.\n", int_value);

        string_value = mpod_snmp_channelGetStatusAsString(obj_ptr, module_slot_index, channel_index);
        if (string_value) {
            printf("ChannelGetStatusAsString() returns %s.\n", string_value);
        }

        int_value = mpod_snmp_channelGetSupervisionBehavior(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetSupervisionBehavior() returns %d.\n", int_value);

        double_value = mpod_snmp_channelGetSupervisionMaxCurrent(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetSupervisionMaxCurrent() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetSupervisionMaxPower(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetSupervisionMaxPower() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetSupervisionMaxTerminalVoltage(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetSupervisionMaxTerminalVoltage() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetSupervisionMinSenseVoltage(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetSupervisionMinSenseVoltage() returns %f.\n", double_value);

        int_value = mpod_snmp_channelGetSwitch(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetSwitch() returns 0x%x.\n", int_value);

        int_value = mpod_snmp_channelGetTripTime(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetTripTime() returns %d.\n", int_value);

        int_value = mpod_snmp_channelGetUserConfig(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetUserConfig() returns 0x%x.\n", int_value);

        double_value = mpod_snmp_channelGetVoltage(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetVoltage() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetVoltageFallRate(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetVoltageFallRate() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetVoltageRiseRate(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetVoltageRiseRate() returns %f.\n", double_value);

        double_value = 0.0005;
        int_value = mpod_snmp_channelSetCurrent(obj_ptr, module_slot_index, channel_index, double_value);
        printf("ChannelSetCurrent() returns %d.\n", int_value);

        double_value = 1.1;
        int_value = mpod_snmp_channelSetCurrentFallRate(obj_ptr, module_slot_index, channel_index, double_value);
        printf("ChannelSetCurrentFallRate() returns %d.\n", int_value);

        double_value = 1.2;
        int_value = mpod_snmp_channelSetCurrentRiseRate(obj_ptr, module_slot_index, channel_index, double_value);
        printf("ChannelSetCurrentRiseRate() returns %d.\n", int_value);

        int_value = 17680;
        int_value = mpod_snmp_channelSetSupervisionBehavior(obj_ptr, module_slot_index, channel_index, int_value);
        printf("ChannelSetSupervisionBehavior() returns %d.\n", int_value);

        double_value = 1e-3;
        int_value = mpod_snmp_channelSetSupervisionMaxCurrent(obj_ptr, module_slot_index, channel_index, double_value);
        printf("ChannelSetSupervisionMaxCurrent() returns %d.\n", int_value);

        double_value = 3.0;
        int_value = mpod_snmp_channelSetSupervisionMaxTerminalVoltage(obj_ptr, module_slot_index, channel_index, double_value);
        printf("ChannelSetSupervisionMaxTerminalVoltage() returns %d.\n", int_value);

        double_value = 1.0;
        int_value = mpod_snmp_channelSetSupervisionMinSenseVoltage(obj_ptr, module_slot_index, channel_index, double_value);
        printf("ChannelSetSupervisionMinSenseVoltage() returns %d.\n", int_value);

        int_value = 1;
        int_value = mpod_snmp_channelSetSwitch(obj_ptr, module_slot_index, channel_index, int_value);
        printf("ChannelSetSwitch() returns %d.\n", int_value);

        int_value = 100;
        int_value = mpod_snmp_channelSetTripTime(obj_ptr, module_slot_index, channel_index, int_value);
        printf("ChannelSetTripTime() returns %d.\n", int_value);

        int_value = 8;
        int_value = mpod_snmp_channelSetUserConfig(obj_ptr, module_slot_index, channel_index, int_value);
        printf("ChannelSetUserConfig() returns %d.\n", int_value);

        double_value = 4.5;
        int_value = mpod_snmp_channelSetVoltage(obj_ptr, module_slot_index, channel_index, double_value);
        printf("ChannelSetVoltage() returns %d.\n", int_value);

        double_value = 1.0;
        int_value = mpod_snmp_channelSetVoltageFallRate(obj_ptr, module_slot_index, channel_index, double_value);
        printf("ChannelSetVoltageFallRate() returns %d.\n", int_value);

        double_value = 2.0;
        int_value = mpod_snmp_channelSetVoltageRiseRate(obj_ptr, module_slot_index, channel_index, double_value);
        printf("ChannelSetVoltageRiseRate() returns %d.\n", int_value);

        /* make sure all data has been written and read to the buffer */
        sleep_ms(3 * thread_polling_interval_ms);

        double_value = mpod_snmp_channelGetCurrent(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetCurrent() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetCurrentFallRate(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetCurrentFallRate() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetCurrentRiseRate(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetCurrentRiseRate() returns %f.\n", double_value);

        int_value = mpod_snmp_channelGetSupervisionBehavior(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetSupervisionBehavior() returns 0x%x.\n", int_value);

        double_value = mpod_snmp_channelGetSupervisionMaxCurrent(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetSupervisionMaxCurrent() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetSupervisionMaxTerminalVoltage(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetSupervisionMaxTerminalVoltage() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetSupervisionMinSenseVoltage(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetSupervisionMinSenseVoltage() returns %f.\n", double_value);

        int_value = mpod_snmp_channelGetSwitch(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetSwitch() returns 0x%x.\n", int_value);

        int_value = mpod_snmp_channelGetTripTime(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetTripTime() returns %f.\n", double_value);

        int_value = mpod_snmp_channelGetUserConfig(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetUserConfig() returns 0x%x.\n", int_value);

        double_value = mpod_snmp_channelGetVoltage(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetVoltage() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetVoltageFallRate(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetVoltageFallRate() returns %f.\n", double_value);

        double_value = mpod_snmp_channelGetVoltageRiseRate(obj_ptr, module_slot_index, channel_index);
        printf("ChannelGetVoltageRiseRate() returns %f.\n", double_value);

    }
    else {

        /* print error messages */
        printf("CrateCheckConnectionIsOk() returns false.\n");
        int_value = mpod_snmp_crateGetConnectionStatus(obj_ptr);
        printf("CrateGetConnectionStatus() returns %d.\n", int_value);
        string_value = mpod_snmp_crateGetConnectionStatusAsString(obj_ptr);
        printf("CrateGetConnectionStatusAsString() returns %s.\n", string_value);
        string_value = mpod_snmp_crateGetErrorString(obj_ptr);
        printf("CrateGetErrorString() returns %s.\n", string_value);

    }

    /* delete object */
    mpod_snmp_delete(obj_ptr);

    mpod_snmp_winsock_cleanup();

#ifdef WIN32
    getchar();
#endif

    return 0;
}
