/*

    Copyright (c) 2018 - 2020 Analog Flavor EURL

    Contact us for support:
        http://www.analogflavor.com

-----------------------------------------------------------------------------
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

    * Neither the name of Analog Flavor EURL nor 
      the names of its contributors may be used to endorse or promote products derived from
      this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
-----------------------------------------------------------------------------

*/

#include "WienerMpodLvHvSnmpPython.h"

#include <Python.h>
#include "structmember.h"

#include <math.h>
#include <iostream>
#include <sstream>

#ifdef WIN32
#include <WinSock2.h>
#endif

//#define DEBUG_PYTHON_INTEGRATION 1
//#define DEBUG_PYTHON 1

#if ( PY_MAJOR_VERSION == 3 )

    #define __LIBRARY_INIT_FUNCTION_NAME__ PyInit_WienerMpodLvHvSnmpPython_3
    #define __LIBRARY__ "WienerMpodLvHvSnmpPython_3"
    static const char* __LIBRARY_NAME__( __LIBRARY__ );

#elif ( PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION == 7 )

    #define __LIBRARY_INIT_FUNCTION_NAME__ initWienerMpodLvHvSnmpPython_2_7
    #define __LIBRARY__ "WienerMpodLvHvSnmpPython_2_7"
    static const char* __LIBRARY_NAME__( __LIBRARY__ );

#endif

#if defined _WIN32 || defined __CYGWIN__
    #define AF_WP_PYTHON_API __declspec(dllexport)
#else
    #if __GNUC__ >= 4
        #define AF_WP_PYTHON_API __attribute__ ((visibility("default")))
    #else
        #define AF_WP_PYTHON_API
    #endif
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifdef AF_VSTUDIO
    PyMODINIT_FUNC __LIBRARY_INIT_FUNCTION_NAME__(void);
#else
#if ( PY_MAJOR_VERSION > 2 )
    AF_WP_PYTHON_API PyObject * __LIBRARY_INIT_FUNCTION_NAME__(void);
#else
    AF_WP_PYTHON_API void __LIBRARY_INIT_FUNCTION_NAME__(void);
#endif
#endif

    static PyObject *_wrap_mpod_snmp_crateConnect(PyObject *self, PyObject *args);// (void *obj_ptr, const char *ip_address, int port_index, const char *community_name, int thread_polling_interval_ms_fast, int thread_polling_interval_ms_intermediate, int thread_polling_interval_ms_slow);
    static PyObject *_wrap_mpod_snmp_crateDisconnect(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateGetConnectionStatus(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateGetConnectionStatusAsString(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateCheckConnectionIsOk(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateGetErrorString(PyObject *self, PyObject *args); // (void *obj_ptr);

    static PyObject *_wrap_mpod_snmp_checkIsMibFileLoaded(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_snmpSetTimeoutInterval(PyObject *self, PyObject *args); // (void *obj_ptr, int interval_in_ms);

    static PyObject *_wrap_mpod_snmp_crateGetNumberOfOccupiedSlots(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateGetOccupiedSlots(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_moduleGetNumberOfChannels(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetChannel(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetIsHvModule(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);

    static PyObject *_wrap_mpod_snmp_channelGetConfigMaxCurrent(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetConfigMaxSenseVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetConfigMaxTerminalVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetCurrent(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetCurrentFallRate(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetCurrentRiseRate(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetIndex(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetMeasurementCurrent(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetMeasurementSenseVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetMeasurementTemperature(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetMeasurementTerminalVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetStatus(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetStatusAsString(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetSupervisionBehavior(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetSupervisionMaxCurrent(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetSupervisionMaxPower(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetSupervisionMaxTerminalVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetSupervisionMinSenseVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetSwitch(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetTripTime(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetUserConfig(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetVoltageFallRate(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelGetVoltageRiseRate(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index);
    static PyObject *_wrap_mpod_snmp_channelSetCurrent(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, double value);
    static PyObject *_wrap_mpod_snmp_channelSetCurrentFallRate(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, double value);
    static PyObject *_wrap_mpod_snmp_channelSetCurrentRiseRate(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, double value);
    static PyObject *_wrap_mpod_snmp_channelSetSupervisionBehavior(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, int value);
    static PyObject *_wrap_mpod_snmp_channelSetSupervisionMaxCurrent(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, double value);
    static PyObject *_wrap_mpod_snmp_channelSetSupervisionMaxTerminalVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, double value);
    static PyObject *_wrap_mpod_snmp_channelSetSupervisionMinSenseVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, double value);
    static PyObject *_wrap_mpod_snmp_channelSetSwitch(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, int value);
    static PyObject *_wrap_mpod_snmp_channelSetTripTime(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, int value);
    static PyObject *_wrap_mpod_snmp_channelSetUserConfig(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, int value);
    static PyObject *_wrap_mpod_snmp_channelSetVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, double value);
    static PyObject *_wrap_mpod_snmp_channelSetVoltageFallRate(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, double value);
    static PyObject *_wrap_mpod_snmp_channelSetVoltageRiseRate(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, unsigned channel_index, double value);
    static PyObject *_wrap_mpod_snmp_crateGetFanSpeed(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateGetNumberOfChannels(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateGetNumberOfGroups(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateGetNumberOfModules(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateGetSerialNumber(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateGetSysMainSwitch(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateGetSysDescr(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateGetSysStatus(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateGetSysStatusAsString(PyObject *self, PyObject *args); // (void *obj_ptr);
    static PyObject *_wrap_mpod_snmp_crateSetFanSpeed(PyObject *self, PyObject *args); // (void *obj_ptr, int value);
    static PyObject *_wrap_mpod_snmp_crateSetSysHardwareReset(PyObject *self, PyObject *args); // (void *obj_ptr, int value);
    static PyObject *_wrap_mpod_snmp_crateSetSysMainSwitch(PyObject *self, PyObject *args); // (void *obj_ptr, int value);
    static PyObject *_wrap_mpod_snmp_groupGetIndex(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned index);
    static PyObject *_wrap_mpod_snmp_groupSetSwitch(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned group_index, int value);
    static PyObject *_wrap_mpod_snmp_moduleGetAuxiliaryMeasurementVoltage0(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetAuxiliaryMeasurementVoltage1(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetDescription(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetEventChannelStatus(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetEventStatus(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetEventStatusAsString(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetHardwareLimitCurrent(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetHardwareLimitVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetIndex(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetMeasurementTemperature0(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetMeasurementTemperature1(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetMeasurementTemperature2(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetMeasurementTemperature3(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetRampSpeedCurrent(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetRampSpeedVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetStatus(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleGetStatusAsString(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_moduleSetDoClear(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, int value);
    static PyObject *_wrap_mpod_snmp_moduleSetRampSpeedCurrent(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, double value);
    static PyObject *_wrap_mpod_snmp_moduleSetRampSpeedVoltage(PyObject *self, PyObject *args); // (void *obj_ptr, unsigned slot_index, double value);

    static PyObject *_wrap_mpod_snmp_snmpGetDescription(PyObject *self, PyObject *args); // (void *obj_ptr, const char *string_oid);
    static PyObject *_wrap_mpod_snmp_snmpGetNextOid(PyObject *self, PyObject *args); // (void *obj_ptr, const char *string_oid);
    static PyObject *_wrap_mpod_snmp_snmpGetValue(PyObject *self, PyObject *args); // (void *obj_ptr, const char *string_oid);
    static PyObject *_wrap_mpod_snmp_snmpGetValueDouble(PyObject *self, PyObject *args); // (void *obj_ptr, const char *string_oid);
    static PyObject *_wrap_mpod_snmp_snmpGetValueInt(PyObject *self, PyObject *args); // (void *obj_ptr, const char *string_oid);
    static PyObject *_wrap_mpod_snmp_snmpGetValueString(PyObject *self, PyObject *args); // (void *obj_ptr, const char *string_oid);
    static PyObject *_wrap_mpod_snmp_snmpSetValueDouble(PyObject *self, PyObject *args); // (void *obj_ptr, const char *string_oid, double value);
    static PyObject *_wrap_mpod_snmp_snmpSetValueInt(PyObject *self, PyObject *args); // (void *obj_ptr, const char *string_oid, int value);
    static PyObject *_wrap_mpod_snmp_snmpTranslateFromOid(PyObject *self, PyObject *args); // (void *obj_ptr, const char *string_oid);
    static PyObject *_wrap_mpod_snmp_snmpTranslateToOid(PyObject *self, PyObject *args); // (void *obj_ptr, const char *string_oid);
    static PyObject *_wrap_mpod_snmp_convertCrateStatusToString(PyObject *self, PyObject *args); // (void *obj_ptr, int integer_status);
    static PyObject *_wrap_mpod_snmp_convertModuleEventStatusToString(PyObject *self, PyObject *args); // (void *obj_ptr, int integer_status);
    static PyObject *_wrap_mpod_snmp_convertModuleStatusToString(PyObject *self, PyObject *args); // (void *obj_ptr, int integer_status);
    static PyObject *_wrap_mpod_snmp_convertChannelStatusToString(PyObject *self, PyObject *args); // (void *obj_ptr, int integer_status, unsigned slot_index);
    static PyObject *_wrap_mpod_snmp_crateGetModule(PyObject *self, PyObject *args); // (void *obj_ptr, int integer_status, unsigned slot_index);

#ifdef __cplusplus
}
#endif

static PyMethodDef af_py_mpod_snmp_methods[] = {
    { (char *)"crateConnect", _wrap_mpod_snmp_crateConnect, METH_VARARGS, "Connects to a Wiener MPOD crate and returns a crate object.\nRequires ip_address, port_index and community_name as arguments. \nThe 3 arguments defining the thread polling intervals are optional."},
    { (char *)"convertCrateStatusToString", _wrap_mpod_snmp_convertCrateStatusToString, METH_VARARGS, "This function converts the integer crate status returned by the crate method crate.getStatus() to a string."},
    { (char *)"convertModuleEventStatusToString", _wrap_mpod_snmp_convertModuleEventStatusToString, METH_VARARGS, "This function converts the integer module event status returned by the module method module.getEventStatus() to a string."},
    { (char *)"convertModuleStatusToString", _wrap_mpod_snmp_convertModuleStatusToString, METH_VARARGS, "This function converts the integer module status returned by the module method module.getStatus() to a string."},
    { (char *)"convertChannelStatusToString", _wrap_mpod_snmp_convertChannelStatusToString, METH_VARARGS, "This function converts the integer channel status returned by the module channel channel.getStatus() to a string."},
    {NULL, NULL, 0, NULL}        /* Sentinel */
};

static WienerMpodLvHvSnmp *_mpod_snmp_get_object_ptr() {
    static WienerMpodLvHvSnmpPython object;
    static bool flag_first_time = false;
    if ( flag_first_time ) {
#ifdef WIN32
        WSADATA WinsockData;
        int res = WSAStartup(MAKEWORD(2, 0), &WinsockData);
#endif
        flag_first_time = false;
    }
    return object.getCratePtr();
}

static PyObject *_mpod_snmp_create_python_vector_from_double_vector(int byte_size, const double *data_double, int first_index, int last_index) {
    PyObject *result = 0;
    PyObject *item = 0;
    int vector_size;
    if (first_index < 0) {
        first_index = 0;
    }
    vector_size = byte_size / sizeof(double);
    if (last_index > vector_size) {
        last_index = vector_size;
    }
    vector_size = last_index - first_index;
    if (data_double && vector_size > 0) {
        result = PyList_New(vector_size);
        data_double = data_double + first_index;
        for ( int i = 0; i < vector_size; i++ ) {
            item = PyFloat_FromDouble(*data_double);
            PyList_SetItem(result, i, item);
            data_double++;
        }
    }
    return result;
}

static PyObject *_mpod_snmp_create_python_vector_from_int_vector(const std::vector<size_t> &vec) {
    PyObject *result = 0;
    PyObject *item = 0;
    int vector_size;
    vector_size = vec.size();
    result = PyList_New(vector_size);
    for ( int i = 0; i < vector_size; i++ ) {
        item = PyLong_FromLong(vec[i]);
        PyList_SetItem(result, i, item);
    }
    return result;
}

static PyObject *_mpod_snmp_create_string_object(const std::string &s) {
    PyObject *result = 0;
    bool flag_is_ascii = true;
    char x;
    unsigned char u;
    /// we need to handle non-ascii strings here
    for ( std::string::const_iterator it = s.begin(); it != s.end(); it++ ) {
        x = *it;
        if ( x == '\n' || x == '\r' || x == '\t' ) {
            continue;
        }
        if ( x >= 127 || x < 32 ) {
            flag_is_ascii = false;
            break;
        }
    }
    if ( flag_is_ascii ) {
#if ( PY_MAJOR_VERSION > 2 )
        result = PyUnicode_FromString(s.c_str());
#else
        result = PyString_FromString(s.c_str());
#endif
    }
    else {
        std::stringstream tmp;
        tmp << "Hex-STRING:" << std::hex;
        for ( std::string::const_iterator it = s.begin(); it != s.end(); it++ ) {
            x = *it;
            u = (unsigned char)x;
            tmp << " " << (size_t)u;
        }
#if ( PY_MAJOR_VERSION > 2 )
        result = PyUnicode_FromString(tmp.str().c_str());
#else
        result = PyString_FromString(tmp.str().c_str());
#endif
    }
    return result;
}

static bool _mpod_snmp_python_check_type(PyObject *obj, PyTypeObject *type)
{
    if (obj) {
        PyObject *objType = PyObject_Type(obj);
        if (objType) {
            if (PyType_Check(objType)) {
                return ((PyTypeObject*)objType == type);
            }
        }
    }
    return false;
}

static void _mpod_snmp_python_free_object(PyObject *obj) {
    freefunc freeFunc = 0;
    if (obj) {
        PyObject *objType = PyObject_Type(obj);
        if (objType) {
            if (PyType_Check(objType)) {
                PyTypeObject* t = (PyTypeObject*)objType;
#if ( PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION > 3 )
                void *f = PyType_GetSlot(t, Py_tp_free);
                if (f) {
                    freeFunc = (freefunc)f;
                }
#else
                freeFunc = t->tp_free;
#endif
                if (freeFunc) {
                    freeFunc(obj);
                }
            }
            Py_DECREF(objType);
        }
    }
}

/// CRATE TYPE
/// this c type corresponds to the below python type
typedef struct {
    PyObject_HEAD
    PyObject *m_address;
} mpod_snmp_c_crate_type;

static PyTypeObject *_mpod_snmp_python_crate_type = 0;

static PyMemberDef _mpod_snmp_python_crate_members[] = {
    {(char*)"address", T_OBJECT, offsetof(mpod_snmp_c_crate_type, m_address), 0, (char*)"crate IP address"},
    {NULL, 0, 0, 0, 0}  /* Sentinel */
};

static PyMethodDef _mpod_snmp_python_crate_methods[] = {
    { (char *)"disconnect", _wrap_mpod_snmp_crateDisconnect, METH_NOARGS, "Disconnects from the currently controlled MPOD crate."},
    { (char *)"getConnectionStatus", _wrap_mpod_snmp_crateGetConnectionStatus, METH_NOARGS, "Returns the connection status."},
    { (char *)"getConnectionStatusAsString", _wrap_mpod_snmp_crateGetConnectionStatusAsString, METH_NOARGS, "Returns the connection status as string."},
    { (char *)"checkConnectionIsOk", _wrap_mpod_snmp_crateCheckConnectionIsOk, METH_NOARGS, "Returns True if the connection has been established and False is an error occurred. "},
    { (char *)"getErrorString", _wrap_mpod_snmp_crateGetErrorString, METH_NOARGS, "The returned string gives a hint on what went wrong in the previous function call."},
    { (char *)"checkIsMibFileLoaded", _wrap_mpod_snmp_checkIsMibFileLoaded, METH_NOARGS, "Returns True if the Wiener MIB file was loaded."},
    { (char *)"getNumberOfOccupiedSlots", _wrap_mpod_snmp_crateGetNumberOfOccupiedSlots, METH_NOARGS, "Returns the number of slots that are occupied by LV or HV modules."},
    { (char *)"getOccupiedSlots", _wrap_mpod_snmp_crateGetOccupiedSlots, METH_NOARGS, "Returns the indices of the slots that are occupied by LV or HV modules."},
    { (char *)"getFanSpeed", _wrap_mpod_snmp_crateGetFanSpeed, METH_VARARGS, ""},
    { (char *)"getNumberOfChannels", _wrap_mpod_snmp_crateGetNumberOfChannels, METH_VARARGS, ""},
    { (char *)"getNumberOfGroups", _wrap_mpod_snmp_crateGetNumberOfGroups, METH_VARARGS, ""},
    { (char *)"getNumberOfModules", _wrap_mpod_snmp_crateGetNumberOfModules, METH_VARARGS, ""},
    { (char *)"getSerialNumber", _wrap_mpod_snmp_crateGetSerialNumber, METH_VARARGS, ""},
    { (char *)"getSysMainSwitch", _wrap_mpod_snmp_crateGetSysMainSwitch, METH_VARARGS, ""},
    { (char *)"getSysDescr", _wrap_mpod_snmp_crateGetSysDescr, METH_VARARGS, ""},
    { (char *)"getSysStatus", _wrap_mpod_snmp_crateGetSysStatus, METH_VARARGS, ""},
    { (char *)"getSysStatusAsString", _wrap_mpod_snmp_crateGetSysStatusAsString, METH_VARARGS, ""},
    { (char *)"setFanSpeed", _wrap_mpod_snmp_crateSetFanSpeed, METH_VARARGS, ""},
    { (char *)"setSysHardwareReset", _wrap_mpod_snmp_crateSetSysHardwareReset, METH_VARARGS, ""},
    { (char *)"setSysMainSwitch", _wrap_mpod_snmp_crateSetSysMainSwitch, METH_VARARGS, ""},
    { (char *)"groupGetIndex", _wrap_mpod_snmp_groupGetIndex, METH_VARARGS, ""},
    { (char *)"groupSetSwitch", _wrap_mpod_snmp_groupSetSwitch, METH_VARARGS, ""},
    { (char *)"snmpSetTimeoutInterval", _wrap_mpod_snmp_snmpSetTimeoutInterval, METH_VARARGS, "Sets the timeout interval for the net-snmp library in milliseconds. The default value is 500 ms."},
    { (char *)"snmpGetDescription", _wrap_mpod_snmp_snmpGetDescription, METH_VARARGS, ""},
    { (char *)"snmpGetNextOid", _wrap_mpod_snmp_snmpGetNextOid, METH_VARARGS, ""},
    { (char *)"snmpGetValue", _wrap_mpod_snmp_snmpGetValue, METH_VARARGS, ""},
    { (char *)"snmpGetValueDouble", _wrap_mpod_snmp_snmpGetValueDouble, METH_VARARGS, ""},
    { (char *)"snmpGetValueInt", _wrap_mpod_snmp_snmpGetValueInt, METH_VARARGS, ""},
    { (char *)"snmpGetValueString", _wrap_mpod_snmp_snmpGetValueString, METH_VARARGS, ""},
    { (char *)"snmpSetValueDouble", _wrap_mpod_snmp_snmpSetValueDouble, METH_VARARGS, ""},
    { (char *)"snmpSetValueInt", _wrap_mpod_snmp_snmpSetValueInt, METH_VARARGS, ""},
    { (char *)"snmpTranslateFromOid", _wrap_mpod_snmp_snmpTranslateFromOid, METH_VARARGS, ""},
    { (char *)"snmpTranslateToOid", _wrap_mpod_snmp_snmpTranslateToOid, METH_VARARGS, ""},
    { (char *)"getModule", _wrap_mpod_snmp_crateGetModule, METH_VARARGS, "Returns a module object for a given slot_index."},
    {NULL, NULL, 0, NULL}  /* Sentinel */
};

static void _mpod_snmp_python_crate_dealloc(PyObject* self) {
    // disconnect
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    if (obj_ptr) {
        obj_ptr->crateDisconnect();
    }
    _mpod_snmp_python_free_object(self);
}

static PyObject *_mpod_snmp_python_crate_repr(mpod_snmp_c_crate_type* self) {
    Py_IncRef(self->m_address);
    return self->m_address;
}

static PyObject *_mpod_snmp_python_crate_str(mpod_snmp_c_crate_type* self) {
    return _mpod_snmp_python_crate_repr(self);
}

static PyObject *_mpod_snmp_python_crate_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
    /// create a new c type
    mpod_snmp_c_crate_type *self;
    self = (mpod_snmp_c_crate_type *)PyType_GenericAlloc(type, 1);
    if (self) {
        self->m_address = 0;
    }
#ifdef DEBUG_PYTHON_INTEGRATION
    std::cout << " _mpod_snmp_python_crate_new : crated crate object at 0x" << std::hex << (size_t)(self) << std::endl;
#endif
    return (PyObject *)self;
}

/// defines the functions for this type
#if ( PY_MAJOR_VERSION == 3 )
static PyType_Slot _mpod_snmp_python_crate_type_slots[] = {
    {Py_tp_doc, (void*)"defines crate objects"},
    {Py_tp_new, (void*)_mpod_snmp_python_crate_new},
    {Py_tp_members, (void*)(&_mpod_snmp_python_crate_members)},
    {Py_tp_methods, (void*)(&_mpod_snmp_python_crate_methods)},
    {Py_tp_str, (void*)_mpod_snmp_python_crate_str},
    {Py_tp_repr, (void*)_mpod_snmp_python_crate_repr},
    {Py_tp_dealloc, (void*)_mpod_snmp_python_crate_dealloc},
    {0, 0}
};

/// the python type is generated based on this specification
static PyType_Spec _mpod_snmp_python_crate_type_spec = {
    __LIBRARY__ ".crate",
    sizeof(mpod_snmp_c_crate_type), // basic size
    0, // item size (indicates types with variable size)
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_DICT_SUBCLASS,
    _mpod_snmp_python_crate_type_slots
};
#else
static PyTypeObject _mpod_snmp_python_crate_type_static = {
    PyVarObject_HEAD_INIT(NULL, 0)
    __LIBRARY__ ".crate",             /* tp_name */
    sizeof(mpod_snmp_c_crate_type),             /* tp_basicsize */
    0,                         /* tp_itemsize */
    (destructor)_mpod_snmp_python_crate_dealloc, /* tp_dealloc */
    0,                         /* tp_print */
    0,                         /* tp_getattr */
    0,                         /* tp_setattr */
    0,                         /* tp_reserved */
    (reprfunc)_mpod_snmp_python_crate_repr,                         /* tp_repr */
    0,                         /* tp_as_number */
    0,                         /* tp_as_sequence */
    0,                         /* tp_as_mapping */
    0,                         /* tp_hash  */
    0,                         /* tp_call */
    (reprfunc)_mpod_snmp_python_crate_str,                         /* tp_str */
    0,                         /* tp_getattro */
    0,                         /* tp_setattro */
    0,                         /* tp_as_buffer */
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_DICT_SUBCLASS,   /* tp_flags */
    "defines crate objects",           /* tp_doc */
    0,                         /* tp_traverse */
    0,                         /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    _mpod_snmp_python_crate_methods,             /* tp_methods */
    _mpod_snmp_python_crate_members,             /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    0,      /* tp_init */
    0,                         /* tp_alloc */
    _mpod_snmp_python_crate_new,                 /* tp_new */
};
#endif

static bool _mpod_snmp_create_crate_python_type(PyObject* python_module) {
    /// here we create the python type
    if (_mpod_snmp_python_crate_type == 0) {
#if ( PY_MAJOR_VERSION == 3 )
        _mpod_snmp_python_crate_type = (PyTypeObject*)(PyType_FromSpec(&_mpod_snmp_python_crate_type_spec));
#else
        _mpod_snmp_python_crate_type = &_mpod_snmp_python_crate_type_static;
#endif
#ifdef DEBUG_PYTHON_INTEGRATION
        std::cout << " _mpod_snmp_create_crate_python_type : crated crate type object at 0x" << std::hex << (size_t)(_mpod_snmp_python_crate_type) << std::endl;
#endif
    }
    if (PyType_Ready(_mpod_snmp_python_crate_type) < 0) {
        return false;
    }
    /// used only for help string generation
    if (_mpod_snmp_python_crate_type) {
        Py_INCREF(_mpod_snmp_python_crate_type);
        PyModule_AddObject(python_module, (char*)"crate", (PyObject *)_mpod_snmp_python_crate_type);
    }
    return true;
}

/// MODULE TYPE
/// this c type corresponds to the below python type
typedef struct {
    PyObject_HEAD
    PyObject *m_name;
    PyObject *m_slotIndex;
} mpod_snmp_c_module_type;

static PyTypeObject *_mpod_snmp_python_module_type = 0;

static PyMemberDef _mpod_snmp_python_module_members[] = {
    {(char*)"name", T_OBJECT, offsetof(mpod_snmp_c_module_type, m_name), 0, (char*)"module name"},
    {(char*)"slot_index", T_OBJECT, offsetof(mpod_snmp_c_module_type, m_slotIndex), 0, (char*)"module slot index"},
    {NULL, 0, 0, 0, 0}  /* Sentinel */
};

static PyMethodDef _mpod_snmp_python_module_methods[] = {
    { (char *)"getNumberOfChannels", _wrap_mpod_snmp_moduleGetNumberOfChannels, METH_VARARGS, "Returns the number of channels that have been detected for the module."},
    { (char *)"getIsHvModule", _wrap_mpod_snmp_moduleGetIsHvModule, METH_VARARGS, "This function retuns True if the module is a HV module."},
    { (char *)"getAuxiliaryMeasurementVoltage0", _wrap_mpod_snmp_moduleGetAuxiliaryMeasurementVoltage0, METH_VARARGS, ""},
    { (char *)"getAuxiliaryMeasurementVoltage1", _wrap_mpod_snmp_moduleGetAuxiliaryMeasurementVoltage1, METH_VARARGS, ""},
    { (char *)"getDescription", _wrap_mpod_snmp_moduleGetDescription, METH_VARARGS, ""},
    { (char *)"getEventChannelStatus", _wrap_mpod_snmp_moduleGetEventChannelStatus, METH_VARARGS, ""},
    { (char *)"getEventStatus", _wrap_mpod_snmp_moduleGetEventStatus, METH_VARARGS, ""},
    { (char *)"getEventStatusAsString", _wrap_mpod_snmp_moduleGetEventStatusAsString, METH_VARARGS, ""},
    { (char *)"getHardwareLimitCurrent", _wrap_mpod_snmp_moduleGetHardwareLimitCurrent, METH_VARARGS, ""},
    { (char *)"getHardwareLimitVoltage", _wrap_mpod_snmp_moduleGetHardwareLimitVoltage, METH_VARARGS, ""},
    { (char *)"getIndex", _wrap_mpod_snmp_moduleGetIndex, METH_VARARGS, ""},
    { (char *)"getMeasurementTemperature0", _wrap_mpod_snmp_moduleGetMeasurementTemperature0, METH_VARARGS, ""},
    { (char *)"getMeasurementTemperature1", _wrap_mpod_snmp_moduleGetMeasurementTemperature1, METH_VARARGS, ""},
    { (char *)"getMeasurementTemperature2", _wrap_mpod_snmp_moduleGetMeasurementTemperature2, METH_VARARGS, ""},
    { (char *)"getMeasurementTemperature3", _wrap_mpod_snmp_moduleGetMeasurementTemperature3, METH_VARARGS, ""},
    { (char *)"getRampSpeedCurrent", _wrap_mpod_snmp_moduleGetRampSpeedCurrent, METH_VARARGS, ""},
    { (char *)"getRampSpeedVoltage", _wrap_mpod_snmp_moduleGetRampSpeedVoltage, METH_VARARGS, ""},
    { (char *)"getStatus", _wrap_mpod_snmp_moduleGetStatus, METH_VARARGS, ""},
    { (char *)"getStatusAsString", _wrap_mpod_snmp_moduleGetStatusAsString, METH_VARARGS, ""},
    { (char *)"setDoClear", _wrap_mpod_snmp_moduleSetDoClear, METH_VARARGS, ""},
    { (char *)"setRampSpeedCurrent", _wrap_mpod_snmp_moduleSetRampSpeedCurrent, METH_VARARGS, ""},
    { (char *)"setRampSpeedVoltage", _wrap_mpod_snmp_moduleSetRampSpeedVoltage, METH_VARARGS, ""},
    { (char *)"getChannel", _wrap_mpod_snmp_moduleGetChannel, METH_VARARGS, "Returns a channel object for a given channel_index."},
    {NULL, NULL, 0, NULL}  /* Sentinel */
};

static void _mpod_snmp_python_module_dealloc(PyObject* self) {
    _mpod_snmp_python_free_object(self);
}

static PyObject *_mpod_snmp_python_module_repr(mpod_snmp_c_module_type* self) {
    Py_IncRef(self->m_name);
    return self->m_name;
}

static PyObject *_mpod_snmp_python_module_str(mpod_snmp_c_module_type* self) {
    return _mpod_snmp_python_module_repr(self);
}

static PyObject *_mpod_snmp_python_module_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
    /// create a new c type
    mpod_snmp_c_module_type *self;
    self = (mpod_snmp_c_module_type *)PyType_GenericAlloc(type, 1);
    if (self) {
        self->m_name = 0;
        self->m_slotIndex = 0;
    }
#ifdef DEBUG_PYTHON_INTEGRATION
    std::cout << " _mpod_snmp_python_module_new : crated module object at 0x" << std::hex << (size_t)(self) << std::endl;
#endif
    return (PyObject *)self;
}

/// defines the functions for this type
#if ( PY_MAJOR_VERSION == 3 )
static PyType_Slot _mpod_snmp_python_module_type_slots[] = {
    {Py_tp_doc, (void*)"defines module objects"},
    {Py_tp_new, (void*)_mpod_snmp_python_module_new},
    {Py_tp_members, (void*)(&_mpod_snmp_python_module_members)},
    {Py_tp_methods, (void*)(&_mpod_snmp_python_module_methods)},
    {Py_tp_str, (void*)_mpod_snmp_python_module_str},
    {Py_tp_repr, (void*)_mpod_snmp_python_module_repr},
    {Py_tp_dealloc, (void*)_mpod_snmp_python_module_dealloc},
    {0, 0}
};

/// the python type is generated based on this specification
static PyType_Spec _mpod_snmp_python_module_type_spec = {
    __LIBRARY__ ".module",
    sizeof(mpod_snmp_c_module_type), // basic size
    0, // item size (indicates types with variable size)
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_DICT_SUBCLASS,
    _mpod_snmp_python_module_type_slots
};
#else
static PyTypeObject _mpod_snmp_python_module_type_static = {
    PyVarObject_HEAD_INIT(NULL, 0)
    __LIBRARY__ ".module",             /* tp_name */
    sizeof(mpod_snmp_c_module_type),             /* tp_basicsize */
    0,                         /* tp_itemsize */
    (destructor)_mpod_snmp_python_module_dealloc, /* tp_dealloc */
    0,                         /* tp_print */
    0,                         /* tp_getattr */
    0,                         /* tp_setattr */
    0,                         /* tp_reserved */
    (reprfunc)_mpod_snmp_python_module_repr,                         /* tp_repr */
    0,                         /* tp_as_number */
    0,                         /* tp_as_sequence */
    0,                         /* tp_as_mapping */
    0,                         /* tp_hash  */
    0,                         /* tp_call */
    (reprfunc)_mpod_snmp_python_module_str,                         /* tp_str */
    0,                         /* tp_getattro */
    0,                         /* tp_setattro */
    0,                         /* tp_as_buffer */
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_DICT_SUBCLASS,   /* tp_flags */
    "defines module objects",           /* tp_doc */
    0,                         /* tp_traverse */
    0,                         /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    _mpod_snmp_python_module_methods,             /* tp_methods */
    _mpod_snmp_python_module_members,             /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    0,      /* tp_init */
    0,                         /* tp_alloc */
    _mpod_snmp_python_module_new,                 /* tp_new */
};
#endif

static bool _mpod_snmp_create_module_python_type(PyObject* python_module) {
    /// here we create the python type
    if (_mpod_snmp_python_module_type == 0) {
#if ( PY_MAJOR_VERSION == 3 )
        _mpod_snmp_python_module_type = (PyTypeObject*)(PyType_FromSpec(&_mpod_snmp_python_module_type_spec));
#else
        _mpod_snmp_python_module_type = &_mpod_snmp_python_module_type_static;
#endif
#ifdef DEBUG_PYTHON_INTEGRATION
        std::cout << " _mpod_snmp_create_module_python_type : moduled module type object at 0x" << std::hex << (size_t)(_mpod_snmp_python_module_type) << std::endl;
#endif
    }
    if (PyType_Ready(_mpod_snmp_python_module_type) < 0) {
        return false;
    }
    /// used only for help string generation
    if (_mpod_snmp_python_module_type) {
        Py_INCREF(_mpod_snmp_python_module_type);
        PyModule_AddObject(python_module, (char*)"module", (PyObject *)_mpod_snmp_python_module_type);
    }
    return true;
}

static PyObject *_mpod_snmp_python_module_create(PyObject *crate, int slot_index) {
    std::stringstream tmp;
    if (_mpod_snmp_python_check_type(crate, _mpod_snmp_python_crate_type) ) {
        PyObject *obj = _mpod_snmp_python_module_new(_mpod_snmp_python_module_type, 0, 0);
        mpod_snmp_c_module_type *module = (mpod_snmp_c_module_type*)obj;
        module->m_slotIndex = PyLong_FromLong(slot_index);
        tmp << "module_" << slot_index;
        module->m_name = _mpod_snmp_create_string_object(tmp.str());
        return obj;
    }
    return 0;
}

static bool _mpod_snmp_python_get_module_slot_index(PyObject *self, int &slot_index) {
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_module_type) ) {
        mpod_snmp_c_module_type *module = (mpod_snmp_c_module_type*)(self);
        if (module->m_slotIndex) {
            if (PyLong_Check(module->m_slotIndex)) {
                slot_index = PyLong_AsLong(module->m_slotIndex);
                return true;
            }
        }
    }
    std::cout << "Failed to retrieve slot_index from object." << std::endl;
    return false;
}




/// CHANNEL TYPE
/// this c type corresponds to the below python type
typedef struct {
    PyObject_HEAD
    PyObject *m_name;
    PyObject *m_slotIndex;
    PyObject *m_channelIndex;
} mpod_snmp_c_channel_type;

static PyTypeObject *_mpod_snmp_python_channel_type = 0;

static PyMemberDef _mpod_snmp_python_channel_members[] = {
    {(char*)"name", T_OBJECT, offsetof(mpod_snmp_c_channel_type, m_name), 0, (char*)"channel name"},
    {(char*)"slot_index", T_OBJECT, offsetof(mpod_snmp_c_channel_type, m_slotIndex), 0, (char*)"module slot index"},
    {(char*)"channel_index", T_OBJECT, offsetof(mpod_snmp_c_channel_type, m_channelIndex), 0, (char*)"channel index"},
    {NULL, 0, 0, 0, 0}  /* Sentinel */
};

static PyMethodDef _mpod_snmp_python_channel_methods[] = {
    { (char *)"getConfigMaxCurrent", _wrap_mpod_snmp_channelGetConfigMaxCurrent, METH_VARARGS, ""},
    { (char *)"getConfigMaxSenseVoltage", _wrap_mpod_snmp_channelGetConfigMaxSenseVoltage, METH_VARARGS, ""},
    { (char *)"getConfigMaxTerminalVoltage", _wrap_mpod_snmp_channelGetConfigMaxTerminalVoltage, METH_VARARGS, ""},
    { (char *)"getCurrent", _wrap_mpod_snmp_channelGetCurrent, METH_VARARGS, ""},
    { (char *)"getCurrentFallRate", _wrap_mpod_snmp_channelGetCurrentFallRate, METH_VARARGS, ""},
    { (char *)"getCurrentRiseRate", _wrap_mpod_snmp_channelGetCurrentRiseRate, METH_VARARGS, ""},
    { (char *)"getIndex", _wrap_mpod_snmp_channelGetIndex, METH_VARARGS, ""},
    { (char *)"getMeasurementCurrent", _wrap_mpod_snmp_channelGetMeasurementCurrent, METH_VARARGS, ""},
    { (char *)"getMeasurementSenseVoltage", _wrap_mpod_snmp_channelGetMeasurementSenseVoltage, METH_VARARGS, ""},
    { (char *)"getMeasurementTemperature", _wrap_mpod_snmp_channelGetMeasurementTemperature, METH_VARARGS, ""},
    { (char *)"getMeasurementTerminalVoltage", _wrap_mpod_snmp_channelGetMeasurementTerminalVoltage, METH_VARARGS, ""},
    { (char *)"getStatus", _wrap_mpod_snmp_channelGetStatus, METH_VARARGS, ""},
    { (char *)"getStatusAsString", _wrap_mpod_snmp_channelGetStatusAsString, METH_VARARGS, ""},
    { (char *)"getSupervisionBehavior", _wrap_mpod_snmp_channelGetSupervisionBehavior, METH_VARARGS, ""},
    { (char *)"getSupervisionMaxCurrent", _wrap_mpod_snmp_channelGetSupervisionMaxCurrent, METH_VARARGS, ""},
    { (char *)"getSupervisionMaxPower", _wrap_mpod_snmp_channelGetSupervisionMaxPower, METH_VARARGS, ""},
    { (char *)"getSupervisionMaxTerminalVoltage", _wrap_mpod_snmp_channelGetSupervisionMaxTerminalVoltage, METH_VARARGS, ""},
    { (char *)"getSupervisionMinSenseVoltage", _wrap_mpod_snmp_channelGetSupervisionMinSenseVoltage, METH_VARARGS, ""},
    { (char *)"getSwitch", _wrap_mpod_snmp_channelGetSwitch, METH_VARARGS, ""},
    { (char *)"getTripTime", _wrap_mpod_snmp_channelGetTripTime, METH_VARARGS, ""},
    { (char *)"getUserConfig", _wrap_mpod_snmp_channelGetUserConfig, METH_VARARGS, ""},
    { (char *)"getVoltage", _wrap_mpod_snmp_channelGetVoltage, METH_VARARGS, ""},
    { (char *)"getVoltageFallRate", _wrap_mpod_snmp_channelGetVoltageFallRate, METH_VARARGS, ""},
    { (char *)"getVoltageRiseRate", _wrap_mpod_snmp_channelGetVoltageRiseRate, METH_VARARGS, ""},
    { (char *)"setCurrent", _wrap_mpod_snmp_channelSetCurrent, METH_VARARGS, ""},
    { (char *)"setCurrentFallRate", _wrap_mpod_snmp_channelSetCurrentFallRate, METH_VARARGS, ""},
    { (char *)"setCurrentRiseRate", _wrap_mpod_snmp_channelSetCurrentRiseRate, METH_VARARGS, ""},
    { (char *)"setSupervisionBehavior", _wrap_mpod_snmp_channelSetSupervisionBehavior, METH_VARARGS, ""},
    { (char *)"setSupervisionMaxCurrent", _wrap_mpod_snmp_channelSetSupervisionMaxCurrent, METH_VARARGS, ""},
    { (char *)"setSupervisionMaxTerminalVoltage", _wrap_mpod_snmp_channelSetSupervisionMaxTerminalVoltage, METH_VARARGS, ""},
    { (char *)"setSupervisionMinSenseVoltage", _wrap_mpod_snmp_channelSetSupervisionMinSenseVoltage, METH_VARARGS, ""},
    { (char *)"setSwitch", _wrap_mpod_snmp_channelSetSwitch, METH_VARARGS, ""},
    { (char *)"setTripTime", _wrap_mpod_snmp_channelSetTripTime, METH_VARARGS, ""},
    { (char *)"setUserConfig", _wrap_mpod_snmp_channelSetUserConfig, METH_VARARGS, ""},
    { (char *)"setVoltage", _wrap_mpod_snmp_channelSetVoltage, METH_VARARGS, ""},
    { (char *)"setVoltageFallRate", _wrap_mpod_snmp_channelSetVoltageFallRate, METH_VARARGS, ""},
    { (char *)"setVoltageRiseRate", _wrap_mpod_snmp_channelSetVoltageRiseRate, METH_VARARGS, ""},
    {NULL, NULL, 0, NULL}  /* Sentinel */
};

static void _mpod_snmp_python_channel_dealloc(PyObject* self) {
    _mpod_snmp_python_free_object(self);
}

static PyObject *_mpod_snmp_python_channel_repr(mpod_snmp_c_channel_type* self) {
    Py_IncRef(self->m_name);
    return self->m_name;
}

static PyObject *_mpod_snmp_python_channel_str(mpod_snmp_c_channel_type* self) {
    return _mpod_snmp_python_channel_repr(self);
}

static PyObject *_mpod_snmp_python_channel_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
    /// create a new c type
    mpod_snmp_c_channel_type *self;
    self = (mpod_snmp_c_channel_type *)PyType_GenericAlloc(type, 1);
    if (self) {
        self->m_name = 0;
        self->m_slotIndex = 0;
        self->m_channelIndex = 0;
    }
#ifdef DEBUG_PYTHON_INTEGRATION
    std::cout << " _mpod_snmp_python_channel_new : crated channel object at 0x" << std::hex << (size_t)(self) << std::endl;
#endif
    return (PyObject *)self;
}

/// defines the functions for this type
#if ( PY_MAJOR_VERSION == 3 )
static PyType_Slot _mpod_snmp_python_channel_type_slots[] = {
    {Py_tp_doc, (void*)"defines channel objects"},
    {Py_tp_new, (void*)_mpod_snmp_python_channel_new},
    {Py_tp_members, (void*)(&_mpod_snmp_python_channel_members)},
    {Py_tp_methods, (void*)(&_mpod_snmp_python_channel_methods)},
    {Py_tp_str, (void*)_mpod_snmp_python_channel_str},
    {Py_tp_repr, (void*)_mpod_snmp_python_channel_repr},
    {Py_tp_dealloc, (void*)_mpod_snmp_python_channel_dealloc},
    {0, 0}
};

/// the python type is generated based on this specification
static PyType_Spec _mpod_snmp_python_channel_type_spec = {
    __LIBRARY__ ".channel",
    sizeof(mpod_snmp_c_channel_type), // basic size
    0, // item size (indicates types with variable size)
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_DICT_SUBCLASS,
    _mpod_snmp_python_channel_type_slots
};
#else
static PyTypeObject _mpod_snmp_python_channel_type_static = {
    PyVarObject_HEAD_INIT(NULL, 0)
    __LIBRARY__ ".channel",             /* tp_name */
    sizeof(mpod_snmp_c_channel_type),             /* tp_basicsize */
    0,                         /* tp_itemsize */
    (destructor)_mpod_snmp_python_channel_dealloc, /* tp_dealloc */
    0,                         /* tp_print */
    0,                         /* tp_getattr */
    0,                         /* tp_setattr */
    0,                         /* tp_reserved */
    (reprfunc)_mpod_snmp_python_channel_repr,                         /* tp_repr */
    0,                         /* tp_as_number */
    0,                         /* tp_as_sequence */
    0,                         /* tp_as_mapping */
    0,                         /* tp_hash  */
    0,                         /* tp_call */
    (reprfunc)_mpod_snmp_python_channel_str,                         /* tp_str */
    0,                         /* tp_getattro */
    0,                         /* tp_setattro */
    0,                         /* tp_as_buffer */
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_DICT_SUBCLASS,   /* tp_flags */
    "defines channel objects",           /* tp_doc */
    0,                         /* tp_traverse */
    0,                         /* tp_clear */
    0,                         /* tp_richcompare */
    0,                         /* tp_weaklistoffset */
    0,                         /* tp_iter */
    0,                         /* tp_iternext */
    _mpod_snmp_python_channel_methods,             /* tp_methods */
    _mpod_snmp_python_channel_members,             /* tp_members */
    0,                         /* tp_getset */
    0,                         /* tp_base */
    0,                         /* tp_dict */
    0,                         /* tp_descr_get */
    0,                         /* tp_descr_set */
    0,                         /* tp_dictoffset */
    0,      /* tp_init */
    0,                         /* tp_alloc */
    _mpod_snmp_python_channel_new,                 /* tp_new */
};
#endif

static bool _mpod_snmp_create_channel_python_type(PyObject* python_module) {
    /// here we create the python type
    if (_mpod_snmp_python_channel_type == 0) {
#if ( PY_MAJOR_VERSION == 3 )
        _mpod_snmp_python_channel_type = (PyTypeObject*)(PyType_FromSpec(&_mpod_snmp_python_channel_type_spec));
#else
        _mpod_snmp_python_channel_type = &_mpod_snmp_python_channel_type_static;
#endif
#ifdef DEBUG_PYTHON_INTEGRATION
        std::cout << " _mpod_snmp_create_channel_python_type : created python channel type object at 0x" << std::hex << (size_t)(_mpod_snmp_python_channel_type) << std::endl;
#endif
    }
    if (PyType_Ready(_mpod_snmp_python_channel_type) < 0) {
        return false;
    }
    /// used only for help string generation
    if (_mpod_snmp_python_channel_type) {
        Py_INCREF(_mpod_snmp_python_channel_type);
        PyModule_AddObject(python_module, (char*)"channel", (PyObject *)_mpod_snmp_python_channel_type);
    }
    return true;
}
static PyObject *_mpod_snmp_python_channel_create(PyObject *module, int channel_index) {
    std::stringstream tmp;
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(module, slot_index) ) {
        PyObject *obj = _mpod_snmp_python_channel_new(_mpod_snmp_python_channel_type, 0, 0);
        mpod_snmp_c_channel_type *channel = (mpod_snmp_c_channel_type*)obj;
        channel->m_slotIndex = PyLong_FromLong(slot_index);
        channel->m_channelIndex = PyLong_FromLong(channel_index);
        tmp << "channel_" << channel_index;
        channel->m_name = _mpod_snmp_create_string_object(tmp.str());
        return obj;
    }
    return 0;
}
static bool _mpod_snmp_python_get_channel_and_slot_index(PyObject *self, int &slot_index, int &channel_index) {
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_channel_type) ) {
        mpod_snmp_c_channel_type *module = (mpod_snmp_c_channel_type*)(self);
        if (module->m_slotIndex && module->m_channelIndex) {
            if (PyLong_Check(module->m_slotIndex) && PyLong_Check(module->m_channelIndex)) {
                slot_index = PyLong_AsLong(module->m_slotIndex);
                channel_index = PyLong_AsLong(module->m_channelIndex);
                return true;
            }
        }
    }
    std::cout << "Failed to retrieve slot_index and channel_index from object." << std::endl;
    return false;
}


static PyObject *_mpod_snmp_python_crate_create(const std::string &address) {
    PyObject *obj = _mpod_snmp_python_crate_new(_mpod_snmp_python_crate_type, 0, 0);
    mpod_snmp_c_crate_type *crate = (mpod_snmp_c_crate_type*)obj;
    crate->m_address = _mpod_snmp_create_string_object(address.c_str());
    return obj;
}

static PyObject *_wrap_mpod_snmp_crateConnect(PyObject *self, PyObject *args) {
    // (void *obj_ptr, const char *ip_address, int port_index, const char *community_name, int thread_polling_interval_ms_fast, int thread_polling_interval_ms_intermediate, int thread_polling_interval_ms_slow);
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    char *ip_address;
    int port_index;
    char *community_name;
    double thread_polling_interval_ms_fast = 1000.0;
    double thread_polling_interval_ms_intermediate = 1000.0;
    double thread_polling_interval_ms_slow = 1000.0;
    PyObject *mpod_crate = 0;
    std::stringstream tmp;
#ifdef DEBUG_PYTHON_INTEGRATION
    std::cout << " << DEBUG_PYTHON_INTEGRATION >> crateConnect() " << std::endl;
#endif
    if ( ! PyArg_ParseTuple(args, "sis|ddd", &ip_address, &port_index, &community_name, &thread_polling_interval_ms_fast, &thread_polling_interval_ms_intermediate, &thread_polling_interval_ms_slow) ) {
        return 0;
    }
    if ( obj_ptr ) {
        // we accept only 1 crate object
        if (obj_ptr->lockAndCheckCrateConnectionIsOk()) {
#ifdef DEBUG_PYTHON
            std::cout << "The current connection must be closed first." << std::endl;
#endif
        }
        else {
            if ( obj_ptr->crateConnect(ip_address, port_index, community_name, thread_polling_interval_ms_fast, thread_polling_interval_ms_intermediate, thread_polling_interval_ms_slow) ) {
                mpod_crate = _mpod_snmp_python_crate_create(obj_ptr->getCompleteIPAddress());
            }
        }
    }
    return mpod_crate;
}
static PyObject *_wrap_mpod_snmp_crateDisconnect(PyObject *self, PyObject *args) {
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        obj_ptr->crateDisconnect();
        return PyBool_FromLong(1);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetConnectionStatus(PyObject *self, PyObject *args) {
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->lockAndGetCrateConnectionStatus();
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetConnectionStatusAsString(PyObject *self, PyObject *args) {
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        std::string result = obj_ptr->lockAndGetCrateConnectionStatusAsString();
        return _mpod_snmp_create_string_object(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateCheckConnectionIsOk(PyObject *self, PyObject *args) {
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->lockAndCheckCrateConnectionIsOk();
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetErrorString(PyObject *self, PyObject *args) {
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        std::string result = obj_ptr->lockAndGetErrorString();
        return _mpod_snmp_create_string_object(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_checkIsMibFileLoaded(PyObject *self, PyObject *args) {
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->checkIsMibFileLoaded();
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_snmpSetTimeoutInterval(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double interval_ms;
        if ( ! PyArg_ParseTuple(args, "d", &interval_ms) ) {
            return 0;
        }
        obj_ptr->snmpSetTimeoutInterval(interval_ms);
        return PyBool_FromLong(1);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetNumberOfOccupiedSlots(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->crateGetOccupiedSlots().size();
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetOccupiedSlots(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        std::vector<size_t> result = obj_ptr->crateGetOccupiedSlots();
        return _mpod_snmp_create_python_vector_from_int_vector(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_convertCrateStatusToString(PyObject *self, PyObject *args){
    int integer_status;
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    if ( ! PyArg_ParseTuple(args, "i", &integer_status) ) {
        return 0;
    }
    std::string result = obj_ptr->convertCrateStatusToString(integer_status);
    return _mpod_snmp_create_string_object(result);
}
static PyObject *_wrap_mpod_snmp_convertModuleEventStatusToString(PyObject *self, PyObject *args){
    int integer_status;
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    if ( ! PyArg_ParseTuple(args, "i", &integer_status) ) {
        return 0;
    }
    std::string result = obj_ptr->convertModuleEventStatusToString(integer_status);
    return _mpod_snmp_create_string_object(result);
}
static PyObject *_wrap_mpod_snmp_convertModuleStatusToString(PyObject *self, PyObject *args){
    int integer_status;
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    if ( ! PyArg_ParseTuple(args, "i", &integer_status) ) {
        return 0;
    }
    std::string result = obj_ptr->convertModuleStatusToString(integer_status);
    return _mpod_snmp_create_string_object(result);
}
static PyObject *_wrap_mpod_snmp_convertChannelStatusToString(PyObject *self, PyObject *args){
    int integer_status;
    int slot_index;
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    if ( ! PyArg_ParseTuple(args, "ii", &integer_status, &slot_index) ) {
        return 0;
    }
    std::string result = obj_ptr->convertChannelStatusToString(integer_status, slot_index);
    return _mpod_snmp_create_string_object(result);
}
static PyObject *_wrap_mpod_snmp_crateGetFanSpeed(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->crateGetFanSpeed();
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetNumberOfChannels(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->crateGetNumberOfChannels();
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetNumberOfGroups(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->crateGetNumberOfGroups();
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetNumberOfModules(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->crateGetNumberOfModules();
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetSerialNumber(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        std::string result = obj_ptr->crateGetSerialNumber();
        return _mpod_snmp_create_string_object(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetSysMainSwitch(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->crateGetSysMainSwitch();
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetSysDescr(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        std::string result = obj_ptr->crateGetSysDescr();
        return _mpod_snmp_create_string_object(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetSysStatus(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->crateGetSysStatus();
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetSysStatusAsString(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        std::string result = obj_ptr->crateGetSysStatusAsString();
        return _mpod_snmp_create_string_object(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateGetModule(PyObject *self, PyObject *args) {
    int slot_index;
    if ( ! PyArg_ParseTuple(args, "i", &slot_index) ) {
        return 0;
    }
    PyObject *module = _mpod_snmp_python_module_create(self, slot_index);
    return module;
}
static PyObject *_wrap_mpod_snmp_crateSetFanSpeed(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        int value;
        if ( ! PyArg_ParseTuple(args, "i", &value) ) {
            return 0;
        }
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->crateSetFanSpeed(value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateSetSysHardwareReset(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        int value;
        if ( ! PyArg_ParseTuple(args, "i", &value) ) {
            return 0;
        }
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->crateSetSysHardwareReset(value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_crateSetSysMainSwitch(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        int value;
        if ( ! PyArg_ParseTuple(args, "i", &value) ) {
            return 0;
        }
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->crateSetSysMainSwitch(value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_groupGetIndex(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        int group_index;
        if ( ! PyArg_ParseTuple(args, "i", &group_index) ) {
            return 0;
        }
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->groupGetIndex(group_index);
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_groupSetSwitch(PyObject *self, PyObject *args){
    if (_mpod_snmp_python_check_type(self, _mpod_snmp_python_crate_type) ) {
        int group_index, value;
        if ( ! PyArg_ParseTuple(args, "ii", &group_index, &value) ) {
            return 0;
        }
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->groupSetSwitch(group_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetNumberOfChannels(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->moduleGetNumberOfChannels(slot_index);
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetIsHvModule(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->moduleGetIsHvModule(slot_index);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetAuxiliaryMeasurementVoltage0(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->moduleGetAuxiliaryMeasurementVoltage0(slot_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetAuxiliaryMeasurementVoltage1(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->moduleGetAuxiliaryMeasurementVoltage1(slot_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetDescription(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        std::string result = obj_ptr->moduleGetDescription(slot_index);
        return _mpod_snmp_create_string_object(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetEventChannelStatus(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->moduleGetEventChannelStatus(slot_index);
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetEventStatus(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->moduleGetEventStatus(slot_index);
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetEventStatusAsString(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        std::string result = obj_ptr->moduleGetEventStatusAsString(slot_index);
        return _mpod_snmp_create_string_object(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetHardwareLimitCurrent(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->moduleGetHardwareLimitCurrent(slot_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetHardwareLimitVoltage(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->moduleGetHardwareLimitVoltage(slot_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetIndex(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->moduleGetIndex(slot_index);
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetMeasurementTemperature0(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->moduleGetMeasurementTemperature0(slot_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetMeasurementTemperature1(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->moduleGetMeasurementTemperature1(slot_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetMeasurementTemperature2(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->moduleGetMeasurementTemperature2(slot_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetMeasurementTemperature3(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->moduleGetMeasurementTemperature3(slot_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetRampSpeedCurrent(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->moduleGetRampSpeedCurrent(slot_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetRampSpeedVoltage(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->moduleGetRampSpeedVoltage(slot_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetStatus(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->moduleGetStatus(slot_index);
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetStatusAsString(PyObject *self, PyObject *args){
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        std::string result = obj_ptr->moduleGetStatusAsString(slot_index);
        return _mpod_snmp_create_string_object(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleGetChannel(PyObject *self, PyObject *args) {
    int channel_index;
    if ( ! PyArg_ParseTuple(args, "i", &channel_index) ) {
        return 0;
    }
    PyObject *channel = _mpod_snmp_python_channel_create(self, channel_index);
    return channel;
}
static PyObject *_wrap_mpod_snmp_moduleSetDoClear(PyObject *self, PyObject *args){
    int value;
    if ( ! PyArg_ParseTuple(args, "i", &value) ) {
        return 0;
    }
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->moduleSetDoClear(slot_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleSetRampSpeedCurrent(PyObject *self, PyObject *args){
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->moduleSetRampSpeedCurrent(slot_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_moduleSetRampSpeedVoltage(PyObject *self, PyObject *args){
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    int slot_index;
    if (_mpod_snmp_python_get_module_slot_index(self, slot_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->moduleSetRampSpeedVoltage(slot_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_snmpGetDescription(PyObject *self, PyObject *args){
    const char *string_oid;
    if ( ! PyArg_ParseTuple(args, "s", &string_oid) ) {
        return 0;
    }
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    std::string result = obj_ptr->snmpGetDescription(string_oid);
    return _mpod_snmp_create_string_object(result);
}
static PyObject *_wrap_mpod_snmp_snmpGetNextOid(PyObject *self, PyObject *args){
    const char *string_oid;
    if ( ! PyArg_ParseTuple(args, "s", &string_oid) ) {
        return 0;
    }
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    std::string result = obj_ptr->snmpGetNextOid(string_oid);
    return _mpod_snmp_create_string_object(result);
}
static PyObject *_wrap_mpod_snmp_snmpGetValue(PyObject *self, PyObject *args){
    const char *string_oid;
    if ( ! PyArg_ParseTuple(args, "s", &string_oid) ) {
        return 0;
    }
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    double double_value = obj_ptr->snmpGetValueDouble(string_oid);
    if ( ! isnan(double_value) ) {
        return PyFloat_FromDouble(double_value);
    }
    int int_value = obj_ptr->snmpGetValueInt(string_oid);
    if ( int_value >= 0 ) {
        return PyLong_FromLong(int_value);
    }
    std::string string_value = obj_ptr->snmpGetValueString(string_oid);
    return _mpod_snmp_create_string_object(string_value);
}
static PyObject *_wrap_mpod_snmp_snmpGetValueDouble(PyObject *self, PyObject *args){
    const char *string_oid;
    if ( ! PyArg_ParseTuple(args, "s", &string_oid) ) {
        return 0;
    }
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    double result = obj_ptr->snmpGetValueDouble(string_oid);
    return PyFloat_FromDouble(result);
}
static PyObject *_wrap_mpod_snmp_snmpGetValueInt(PyObject *self, PyObject *args){
    const char *string_oid;
    if ( ! PyArg_ParseTuple(args, "s", &string_oid) ) {
        return 0;
    }
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    int result = obj_ptr->snmpGetValueInt(string_oid);
    return PyLong_FromLong(result);
}
static PyObject *_wrap_mpod_snmp_snmpGetValueString(PyObject *self, PyObject *args){
    const char *string_oid;
    if ( ! PyArg_ParseTuple(args, "s", &string_oid) ) {
        return 0;
    }
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    std::string result = obj_ptr->snmpGetValueString(string_oid);
    return _mpod_snmp_create_string_object(result);
}
static PyObject *_wrap_mpod_snmp_snmpSetValueDouble(PyObject *self, PyObject *args){
    const char *string_oid;
    double value;
    if ( ! PyArg_ParseTuple(args, "sd", &string_oid, &value) ) {
        return 0;
    }
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    int result = obj_ptr->snmpSetValueDouble(string_oid, value);
    return PyLong_FromLong(result);
}
static PyObject *_wrap_mpod_snmp_snmpSetValueInt(PyObject *self, PyObject *args){
    const char *string_oid;
    int value;
    if ( ! PyArg_ParseTuple(args, "si", &string_oid, &value) ) {
        return 0;
    }
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    int result = obj_ptr->snmpSetValueInt(string_oid, value);
    return PyLong_FromLong(result);
}
static PyObject *_wrap_mpod_snmp_snmpTranslateFromOid(PyObject *self, PyObject *args){
    const char *string_oid;
    if ( ! PyArg_ParseTuple(args, "s", &string_oid) ) {
        return 0;
    }
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    std::string result = obj_ptr->snmpTranslateFromOid(string_oid);
    return _mpod_snmp_create_string_object(result);
}
static PyObject *_wrap_mpod_snmp_snmpTranslateToOid(PyObject *self, PyObject *args){
    const char *string_oid;
    if ( ! PyArg_ParseTuple(args, "s", &string_oid) ) {
        return 0;
    }
    WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
    std::string result = obj_ptr->snmpTranslateToOid(string_oid);
    return _mpod_snmp_create_string_object(result);
}
static PyObject *_wrap_mpod_snmp_channelGetConfigMaxCurrent(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetConfigMaxCurrent(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetConfigMaxSenseVoltage(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetConfigMaxSenseVoltage(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetConfigMaxTerminalVoltage(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetConfigMaxTerminalVoltage(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetCurrent(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetCurrent(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetCurrentFallRate(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetCurrentFallRate(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetCurrentRiseRate(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetCurrentRiseRate(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetIndex(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->channelGetIndex(slot_index, channel_index);
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetMeasurementCurrent(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetMeasurementCurrent(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetMeasurementSenseVoltage(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetMeasurementSenseVoltage(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetMeasurementTemperature(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetMeasurementTemperature(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetMeasurementTerminalVoltage(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetMeasurementTerminalVoltage(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetStatus(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->channelGetStatus(slot_index, channel_index);
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetStatusAsString(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        std::string result = obj_ptr->channelGetStatusAsString(slot_index, channel_index);
        return _mpod_snmp_create_string_object(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetSupervisionBehavior(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->channelGetSupervisionBehavior(slot_index, channel_index);
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetSupervisionMaxCurrent(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetSupervisionMaxCurrent(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetSupervisionMaxPower(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetSupervisionMaxPower(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetSupervisionMaxTerminalVoltage(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetSupervisionMaxTerminalVoltage(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetSupervisionMinSenseVoltage(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetSupervisionMinSenseVoltage(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetSwitch(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->channelGetSwitch(slot_index, channel_index);
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetTripTime(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->channelGetTripTime(slot_index, channel_index);
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetUserConfig(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        int result = obj_ptr->channelGetUserConfig(slot_index, channel_index);
        return PyLong_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetVoltage(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetVoltage(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetVoltageFallRate(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetVoltageFallRate(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelGetVoltageRiseRate(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        double result = obj_ptr->channelGetVoltageRiseRate(slot_index, channel_index);
        return PyFloat_FromDouble(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetCurrent(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetCurrent(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetCurrentFallRate(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetCurrentFallRate(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetCurrentRiseRate(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetCurrentRiseRate(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetSupervisionBehavior(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetSupervisionBehavior(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetSupervisionMaxCurrent(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetSupervisionMaxCurrent(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetSupervisionMaxTerminalVoltage(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetSupervisionMaxTerminalVoltage(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetSupervisionMinSenseVoltage(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetSupervisionMinSenseVoltage(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetSwitch(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetSwitch(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetTripTime(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetTripTime(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetUserConfig(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetUserConfig(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetVoltage(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetVoltage(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetVoltageFallRate(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetVoltageFallRate(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}
static PyObject *_wrap_mpod_snmp_channelSetVoltageRiseRate(PyObject *self, PyObject *args){
    int slot_index, channel_index;
    double value;
    if ( ! PyArg_ParseTuple(args, "d", &value) ) {
        return 0;
    }
    if (_mpod_snmp_python_get_channel_and_slot_index(self, slot_index, channel_index) ) {
        WienerMpodLvHvSnmp *obj_ptr = _mpod_snmp_get_object_ptr();
        bool result = obj_ptr->channelSetVoltageRiseRate(slot_index, channel_index, value);
        return PyBool_FromLong(result);
    }
    return 0;
}

#if ( PY_MAJOR_VERSION > 2 )

static PyModuleDef af_py_mpod_snmp_module = {
    PyModuleDef_HEAD_INIT,
    __LIBRARY_NAME__,
    "This library allows to control a Wiener MPOD crate over SNMP.\nThe current version only supports a single crate.\nThe undocumented methods and functions are explained in the document mpod-snmp-guide.pdf.",
    -1,
    af_py_mpod_snmp_methods,
    NULL,
    NULL,
    NULL,
    NULL
};

#endif


#if ( PY_MAJOR_VERSION > 2 )
PyObject * __LIBRARY_INIT_FUNCTION_NAME__(void)
#else
void __LIBRARY_INIT_FUNCTION_NAME__(void)
#endif
{
#ifdef DEBUG_PYTHON_INTEGRATION
    std::cout << " << DEBUG_PYTHON_INTEGRATION >> initializing python library \"" << __LIBRARY_NAME__ << "\"" << std::endl;
#endif

#if ( PY_MAJOR_VERSION > 2 )
    PyObject* mod = PyModule_Create(&af_py_mpod_snmp_module);
#else
    PyObject* mod = Py_InitModule3(__LIBRARY_NAME__, af_py_mpod_snmp_methods,
                                   "This library allows to control a Wiener MPOD crate over SNMP.\nThe current version only supports a single crate.\nThe undocumented methods and functions are explained in the document mpod-snmp-guide.pdf.");
#endif

    _mpod_snmp_create_crate_python_type(mod);
    _mpod_snmp_create_module_python_type(mod);
    _mpod_snmp_create_channel_python_type(mod);

    /// this initializes the library
    _mpod_snmp_get_object_ptr();

#if ( PY_MAJOR_VERSION > 2 )
    return mod;
#endif
}

WienerMpodLvHvSnmpPython::WienerMpodLvHvSnmpPython()
{
}

WienerMpodLvHvSnmpPython::~WienerMpodLvHvSnmpPython()
{
    m_crate.crateDisconnect();
}



